/*******************************************************************************
 * Copyright 2011 John Casey
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.commonjava.tensor.ispn.data.store;

import java.util.HashSet;
import java.util.Set;

import javax.annotation.PostConstruct;
import javax.enterprise.context.ApplicationScoped;
import javax.inject.Inject;

import org.apache.maven.graph.effective.EProjectGraph;
import org.apache.maven.graph.effective.EProjectNet;
import org.apache.maven.graph.effective.EProjectRelationships;
import org.apache.maven.graph.effective.ref.EProjectKey;
import org.commonjava.tensor.data.IndexManager;
import org.commonjava.tensor.data.TensorDataException;
import org.commonjava.tensor.data.idx.AllByTargetIndex;
import org.commonjava.tensor.data.idx.ChildrenOfIndex;
import org.commonjava.tensor.data.idx.ParentOfIndex;
import org.commonjava.tensor.data.store.TensorDataStore;
import org.infinispan.Cache;
import org.infinispan.manager.CacheContainer;

@ApplicationScoped
public class InfinispanTensorDataStore
    implements TensorDataStore
{

    @Inject
    private CacheContainer container;

    private Cache<EProjectKey, EProjectRelationships> effectiveRelationships;

    private Cache<EProjectKey, EProjectGraph> graphs;

    @PostConstruct
    public void injectCaches()
    {
        graphs = container.getCache( GRAPH_CACHE );
        graphs.start();

        effectiveRelationships = container.getCache( PROJECT_RELATIONSHIPS_CACHE );
        effectiveRelationships.start();
    }

    public InfinispanTensorDataStore()
    {
    }

    public InfinispanTensorDataStore( final CacheContainer container, final IndexManager indexManager,
                                      final ParentOfIndex parentOfIndex, final ChildrenOfIndex childrenOfIndex,
                                      final AllByTargetIndex allByTargetIndex )
    {
        this.graphs = container.getCache( GRAPH_CACHE );
        this.graphs.start();

        this.effectiveRelationships = container.getCache( PROJECT_RELATIONSHIPS_CACHE );
        this.effectiveRelationships.start();
    }

    @Override
    public EProjectRelationships storeRelationshipSet( final EProjectRelationships relationships, final boolean force )
        throws TensorDataException
    {
        if ( force )
        {
            return effectiveRelationships.put( relationships.getKey(), relationships );
        }
        else
        {
            return effectiveRelationships.putIfAbsent( relationships.getKey(), relationships );
        }
    }

    @Override
    public EProjectNet storeGraph( final EProjectGraph graph, final boolean force )
        throws TensorDataException
    {
        if ( force )
        {
            return graphs.put( graph.getKey(), graph );
        }
        else
        {
            return graphs.putIfAbsent( graph.getKey(), graph );
        }
    }

    @Override
    public boolean containsRelationshipSet( final EProjectKey key )
    {
        return effectiveRelationships.containsKey( key );
    }

    @Override
    public boolean containsGraph( final EProjectKey key )
    {
        return graphs.containsKey( key );
    }

    @Override
    public EProjectGraph getGraph( final EProjectKey key )
    {
        return graphs.get( key );
    }

    @Override
    public EProjectRelationships getRelationshipSet( final EProjectKey key )
    {
        return effectiveRelationships.get( key );
    }

    @Override
    public Set<EProjectGraph> getAllGraphs()
    {
        return new HashSet<EProjectGraph>( graphs.values() );
    }

    @Override
    public Set<EProjectRelationships> getAllRelationshipSets()
    {
        return new HashSet<EProjectRelationships>( effectiveRelationships.values() );
    }

    @Override
    public Set<EProjectKey> getGraphKeys()
    {
        return graphs.keySet();
    }

    @Override
    public Set<EProjectKey> getRelationshipSetKeys()
    {
        return effectiveRelationships.keySet();
    }

}
